/***************************************************************************
                          mast.cpp  -  The main mast implementation file
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma implementation
#define __SUCS 1
//#define DEBUG_MESSAGES 1

#include <iostream>
#include <map>
//#include <typeinfo>

using namespace std;

#include <stdio.h>

#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "mast.h"

#include "thread.h"
#include "link.h"

using namespace sucs;

Cleaner<Link> Mast::theCleaner;

Mast::Mast(int newPort) : SignalHandler(), Thread("Listener")
{
	if(!theCleaner.isRunning())
		theCleaner.start();
	if(newPort > -1) thePort = newPort;
	theListening = false;
}

Mast::~Mast()
{
}

void Mast::handler(int)
{
	theGotIO.broadcast(false);
}

void Mast::tryAccept()
{
	fcntl(theSocket, F_SETFL, O_NONBLOCK);
	int s;
	while(1)
	{	s = ::accept(theSocket, NULL, 0);
		if(s > -1)
		{	Link *newLink = newArriver();
			newLink->setMast(this);
			newLink->adopt(s);
			theLinks.push_back(newLink);
		}
		else
			break;
	}
	fcntl(theSocket, F_SETFL, O_ASYNC);
}

const bool Mast::start(int newPort)
{
	theQuitWaiter.engage();
	theQuitHandler.engage();
	if(!bind(newPort))
		return false;
	if(!listen())
		return false;
	return true;
}

void Mast::quit()
{
//	cerr << "Initiating quit(). Stopping listener..." << endl << flush;
	Thread::cancel();
	unhandle(SIGIO);
//	ignore(SIGIO);
	Thread::waitForDeath();
	theListening = false;
//	cerr << "Signalling handler..." << endl << flush;
	theQuitHandler.signal();
//	cerr << "Signalled (" << theQuitHandler << ")." << endl << flush;
}

void Mast::waitForQuit()
{
	theQuitWaiter.hop();
}

void Mast::handleQuit()
{
	while(!theQuitHandler.wait());
	closeAll();
	deleteAll();
	theQuitWaiter.signal();
}

void Mast::safeDelete(Link *c)
{
	theLinksAccess.wait();
	for(list<Link *>::iterator j = theLinks.begin(); j != theLinks.end(); j++)
		if(*j == c)
		{	theLinks.erase(j);
			theCleaner.deleteObject(c);
			break;
		}
	theLinksAccess.signal();
}

void Mast::deleteAll()
{
	theLinksAccess.wait();
	for(list<Link *>::iterator j = theLinks.begin(); j != theLinks.end(); j++)
		delete *j;
	theLinks.clear();
	theLinksAccess.signal();
}

void Mast::closeAll()
{
	theLinksAccess.wait();
	for(list<Link *>::iterator j = theLinks.begin(); j != theLinks.end(); j++)
		(*j)->disconnect();
	theLinksAccess.signal();
}

const bool Mast::bind(int newPort)
{
	theSocket = socket(AF_INET, SOCK_STREAM, 0);

	if(newPort > -1) thePort = newPort;
	sockaddr_in taddr;
	taddr.sin_family = AF_INET;
	taddr.sin_port = htons(thePort);
	taddr.sin_addr.s_addr = INADDR_ANY;

	// potential mem man concern here - is sockaddr allowed to be destroyed?
	if(::bind(theSocket, (struct sockaddr *)&taddr, sizeof(struct sockaddr)))
		return false;
	return true;
}

void Mast::body()
{
	while(1)
	{
		theGotIO.wait();
		tryAccept();
	}
}

void Mast::ghost()
{
	// because we use a deffered cancellation, we can be sure that we have the lock.
	theGotIO.unlock();
}

const bool Mast::listen()
{
	if(theListening) return false;
	
	if(::listen(theSocket, 10))
		return false;
	
	Thread::start();
	
	::fcntl(theSocket, F_SETOWN, getpid());
	::fcntl(theSocket, F_SETFL, O_ASYNC);
	handle(SIGIO);
	
	return true;
}
