/***************************************************************************
                          factorymanager.h
    Factory manager class for handling multiple factories.
                             -------------------
    begin                : Wed Jul 17 2002
    copyright            : (C) 2002 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __FACTORYMANAGER_H
#define __FACTORYMANAGER_H

#pragma interface

#include <iostream>
#include <string>
#include <map>
#include <vector>
using namespace std;

#include <glob.h>

#include "factory.h"
using namespace sucs;

namespace sucs
{

template<class Base> class FactoryManager
{
	map<string, Factory<Base> > theFactories;
	vector<string> theNames;
	
	void loadLibrary(const string &theFile);
	void loadLibraries(const string &thePath);
	
public:
	void reloadAll(const string &thePath)
	{
		theFactories.clear();
		theNames.clear();
		loadLibraries(thePath);
	}

	void reloadAll(const vector<string> &thePaths)
	{
		theFactories.clear();
		theNames.clear();
		for(int i = 0; i < thePaths.size(); i++)
			loadLibraries(thePaths[i]);
	}
	
	const vector<string> &getLoaded() { return theNames; }
	const bool isLoaded(const string &theName) { return theFactories.count(theName); }

	Base *operator[](const string &theName)
	{
		if(theFactories.count(theName))
			return theFactories[theName].createInstance();
		return 0;
	}
};

template<class Base> void FactoryManager<Base>::loadLibrary(const string &theFile)
{
	Factory<Base> newFactory(theFile);
	if(!newFactory.isOpen()) return;
	if(theFactories.find(newFactory.getID()) != theFactories.end())
		if(newFactory.getVersion() <= theFactories[newFactory.getID()].getVersion())
			return;
	theFactories[newFactory.getID()] = newFactory;
	theNames.push_back(newFactory.getID());
}

template<class Base> void FactoryManager<Base>::loadLibraries(const string &thePath)
{
	glob_t theGlob;
	glob(thePath.c_str(), 0, 0, &theGlob);
	for(unsigned i = 0; i < theGlob.gl_pathc; i++)
		loadLibrary(theGlob.gl_pathv[i]);
	globfree(&theGlob);
}

}

#endif
