//
// C++ Implementation: signallistener
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <iostream>
using namespace std;

#include <signal.h>

#include "thread.h"
#include "signalhandler.h"
#include "signallistener.h"
using namespace sucs;

namespace sucs
{

SignalListener *SignalListener::theOne = NULL;

SignalListener::SignalListener()
{
}

SignalListener::~SignalListener()
{
}

void SignalListener::set(int signal, SignalHandler *handler)
{
/*	struct sigaction *act = new struct sigaction;
	act->sa_sigaction = actualHandler;
	act->sa_mask = 0;
	act->sa_flags = SA_SIGINFO;
	sigaction(a, act, NULL);
	delect act;
*/
	tdbg("SignalListener::set: Object " << handler << " is handling signal " << signal << "." << endl);
	if(!theHandlers.count(signal))
		::signal(signal, actualHandler);
	theHandlers.insert(make_pair<int, SignalHandler *>(signal, handler));
}

void SignalListener::reset(int signal, SignalHandler *handler)
{
	tdbg("SignalListener::reset: Object " << handler << " is not handling signal " << signal << "." << endl);

	// erase the signal handler from the handlers list
	for(multimap<int, SignalHandler *>::iterator i = theHandlers.lower_bound(signal); i != theHandlers.upper_bound(signal); i++)
		if((*i).second == handler)
		{	theHandlers.erase(i);
			break;
		}
	
	// if no signals are being handled, resort to the default action
	if(!theHandlers.count(signal))
		::signal(signal, SIG_DFL);
}

void SignalListener::ignore(int signal)
{
	tdbg("SignalListener::ignore: Ignoring signal " << signal << "." << endl);
	::signal(signal, SIG_IGN);
	for(multimap<int, SignalHandler *>::iterator i = theHandlers.lower_bound(signal); i != theHandlers.upper_bound(signal); i++)
		(*i).second->unregister(signal);
	theHandlers.erase(signal);
}

void sucs::actualHandler(int signal)
{
//	cerr << "actualHandler: Received signal " << signal << "." << endl << flush;
	SignalListener *l = SignalListener::signalListener();
	for(multimap<int, SignalHandler *>::iterator i = l->theHandlers.lower_bound(signal); i != l->theHandlers.upper_bound(signal); i++)
		(*i).second->handler(signal);
	::signal(signal, actualHandler);
}

};
