/***************************************************************************
                          buffer.cpp  -  The main Buffer implementation file
                             -------------------
    begin                : Fri 17 January 2003
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#pragma implementation

#include <cmath>
using namespace std;

#ifndef PI
 #ifdef M_PI
  #define PI M_PI
 #else
  #define PI 3.14159265358979323846
 #endif
#endif

#include "buffer.h"
using namespace sucs;

void Buffer::copyFromShort(unsigned numSamples, signed short samples[], float freq)
{
	theSamples.resize(numSamples);
	if(samples)
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = float(samples[i]) / 32767.0;
	else
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = 0;
	theFrequency = freq;
}

void Buffer::copyFromLong(unsigned numSamples, signed long samples[], float freq)
{
	theSamples.resize(numSamples);
	if(samples)
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = float(samples[i]) / 2147483647.0;
	else
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = 0;
	theFrequency = freq;
}

Buffer::Buffer(unsigned numSamples, float samples[], float freq)
{
	theSamples.resize(numSamples);
	if(samples)
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = samples[i];
	else
		for(unsigned i = 0; i < numSamples; i++)
			theSamples[i] = 0;
	theFrequency = freq;
}

Buffer::Buffer(float freq, float secs)
{
	theFrequency = freq;
	theSamples.resize(int(secs * freq));
}

Buffer::~Buffer()
{
}

void Buffer::applyHanning()
{
	for(unsigned i = 0; i < theSamples.size(); i++)
		theSamples[i] = theSamples[i] * cos( (((i / float(theSamples.size())) - 0.5) * PI + 1) / 2 );
}

Buffer Buffer::subBuffer(unsigned index, unsigned length) const
{
	if(!length || length > theSamples.size()) length = theSamples.size();
	Buffer tmp(length, NULL, theFrequency);
	for(int i = 0; i < length; i++)
		tmp[i] = theSamples[i + index];
	return tmp;
}

void Buffer::clear()
{
	for(int i = 0; i < theSamples.size(); i++)
		theSamples[i] = short(0);
}

void Buffer::addPureTone(float freq, float mag)
{
	float a = theFrequency / freq;
	for(int i = 0; i < theSamples.size(); i++)
	{	float wouldBe = theSamples[i] + sin(2 * PI * i / a) * mag;
		if(wouldBe < 1 && wouldBe > -1)
			theSamples[i] = wouldBe;
		else if(wouldBe >= 1)
			theSamples[i] = 1;
		else
			theSamples[i] = -1;
	}
}

void Buffer::resizeForFrequency(float freq)
{
	if(!theFrequency) return;
	theSamples.resize(int(ceil(log(theFrequency / freq) / log(2.0))));
}
