/***************************************************************************
                          ssmatrix.cpp  -  The main Self-Similarity matrix
			    implementation file
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#pragma implementation

#include <iostream>
#include <cmath>
using namespace std;

#include "ssmatrix.h"
using namespace sucs;

SSMatrix::SSMatrix(int newSize, float gapTime)
{
	resize(newSize);
	upToDate = false;
	theGapTime = gapTime;
}

SSMatrix::~SSMatrix()
{
}

void SSMatrix::resize(unsigned newSize)
{
	theData.resize(newSize);
	theMatrix.resize(newSize);
	for(int i = 0; i < newSize; i++)
		theMatrix[i].resize(newSize);
	numPoints = 0;
}

void SSMatrix::newPoint(const Spectrum &newSpectrum)
{
	// move all old data. TODO: easier way of doing this?
	for(int i = 1; i < theMatrix.size(); i++)
	{	theData[i - 1] = theData[i];
		for(int j = 1; j < theMatrix.size(); j++)
			theMatrix[i - 1][j - 1] = theMatrix[i][j];
	}

	// insert new data
	float min = theMatrix[0][0], max = theMatrix[0][0];
	theData.back() = newSpectrum;

	// calculate new ss matrix row/column
	for(int j = 0; j < theMatrix.size(); j++)
	{	float d = theData[j].diffByCosine(newSpectrum);
		theMatrix.back()[j] = theMatrix[j].back() = d;
		for(int i = 0; i < theMatrix.size(); i++)
			if(theMatrix[j][i] > max) max = theMatrix[j][i];
			else if(theMatrix[j][i] < min) min = theMatrix[j][i];
	}

	theOffset = min;
	theMagnitude = max - min;

	numPoints++;
	upToDate = false;
}

void SSMatrix::clear()
{
	// insert clear data
	for(int i = 0; i < theMatrix.size(); i++)
		for(int j = 0; j < theMatrix.size(); j++)
			theMatrix[i][j] = 0;
	numPoints = 0;
	upToDate = false;
}

void SSMatrix::newPopulation(vector<Spectrum *> theSpectra)
{
	clear();

	// insert new data
	for(int i = 0; i < theSpectra.size(); i++)
		theData[i] = *(theSpectra[i]);

	// clear array caches
	for(int i = 0; i < theData.size(); i++)
		theData[i].killArray();

	// calculate new ss matrix row/column
	for(int i = 0; i < theSpectra.size(); i++)
		for(int j = 0; j < theSpectra.size(); j++)
			if(i >= j)
				theMatrix[i][j] = theMatrix[j][i] = theData[i].diffByCosine(theData[j]);
	numPoints = theSpectra.size();
	upToDate = false;
}

void SSMatrix::updateBeatSpectrum()
{
	theBeatSpectrum.resize(theMatrix.size() / 2);
	for(unsigned i = 0; i < theBeatSpectrum.size(); i++)
		theBeatSpectrum[i] = 0;
	//*****************
	// PROBABLY OK
	theBeatSpectrum.setBaseTime(theGapTime);
	//*********************

	for(unsigned i = 0; i < theMatrix.size(); i++)
		for(unsigned j = 0; j < theMatrix.size(); j++)
		{	// TODO: check if theBeatSpectrum.size() should really be theBeatSpectrum.size() - 1 (for proper inverse)
			int band = theBeatSpectrum.size() - ((i - j + theMatrix.size()) % theMatrix.size());
			if(band >= 0 && band < theBeatSpectrum.size())
				theBeatSpectrum[band] += (theMatrix[i][j] / theMatrix.size());
		}

	theBeatSpectrum.killArray();

	upToDate = true;
}
